import clozeTemplate from './cloze.template.html'

/* @ngInject */
let clozeComponent = {
  template: clozeTemplate,
  bindings: {
    questionsId: '=',
    sid: '='
  },
  controller: ($scope, $compile, $element, $rootScope, $location, $http, $state, $stateParams, $localStorage, $timeout, $log, xxtea, exerciseService, $cookies, appConstants, popupService) => {

    $scope.token = 'Bearer ' + $cookies.get('API_TOKEN')
    /**
         * Error callback for HTTP response
         * @param {object} response The HTTP response object
         * @returns {void}
         */
    function errorCallback(response) {
      if (response.status === 401) {
        $log.error({ status: 'false', data: 'Bad token.' })
      } else if (response.status === 404) {
        $log.error({ status: 'false', data: 'Bad request.' })
      } else if (response.status === 405) {
        $log.error({ status: 'false', data: 'Method not allowed.' })
      } else {
        $log.error({ status: 'false', data: response.statusText })
      }

    };

    $scope.queans = {}
    $scope.step = 0 // Current question (step)
    $scope.attempt = 0 // Attempt number
    $scope.overallCorrectAnswers = 0
    $scope.allAnswers = [] // All user answers
    $scope.attempts = [] // All user attempts
    $scope.correct = false // Is user answer correct
    $scope.queans = {} // User answers bindings
    $scope.tryAgain = false // Is in try again state
    $scope.finalAttempt = false // Is in final attempt state
    $scope.time_finished = '' // Time when user started the exercise
    $scope.time_attempted = Math.round(new Date().getTime() / 1000) // Time when the user finished the exercise

    // Popup
    $scope.popupVisible = false
    let popupVisibilityChange = $rootScope.$on('popup.changed', function (elem, data) {
      $scope.popupVisible = data.active
    })

    /**
         * Switch to next question or finish exercise if the question (step) is last
         * @returns {void}
         */
    $scope.nextStep = () => {
      if ($scope.step + 1 === $scope.exercise.interaction.questions.length) {
        $scope.time_finished = Math.round(new Date().getTime() / 1000)
        $scope.finishExercise()
      } else {
        popupService.hide()
        $scope.step++
        $scope.correct = false
        $scope.attempt = 0
        $scope.tryAgain = false
        $scope.finalAttempt = false
        $scope.attempts = []
        $scope.validation = []
      }

      $scope.currentActiveQuestion = $scope.step + 1
      $localStorage.currentActiveQuestion = $scope.step + 1
    }

    /**
         * Setting up the exercise
         * @param {object} exerciseData exercise object
         * @returns {void}
         */
    function setupExercise(exerciseData) {
      $scope.exercise = exerciseData
      $scope.question = $scope.exercise.interaction.questions[$scope.step]
      $scope.answers = $scope.exercise.interaction.correct.answers // Get answers, old $scope.exercise.interaction.correct[$scope.step].answers
      $scope.wordlist = $scope.question.wordlist.join(' - ')
      $localStorage.questionCount = $scope.exercise.interaction.questions.length // Get questions count
    }

    /**
         * Checks all answers and show the feedback in popup directive
         * @returns {void}
         */
    $scope.check = () => {
      const regexpSym = /(['"`.,|()!?:;])/g
      if ($scope.question.answered === true) {
        popupService.show('You have already answered this question!', 'semi-smile', false)
      } else {
        $scope.userAnswers = $scope.queans[$scope.step]
        $scope.validation = []
        $scope.postValidation = []
        $scope.correctAnswers = []
        $scope.postAnswers = []
        $scope.attempt++

        let cQuestion = angular.isArray($scope.exercise.interaction.correct) ?
          $scope.exercise.interaction.correct.find((q) => { return $scope.question.question_id === q.question_id }) : $scope.exercise.interaction.correct
        if (cQuestion) {
          let correctAnswers = cQuestion.answers

          angular.forEach(correctAnswers, (v, k) => {
            let userAnswer = $scope.userAnswers ? $scope.userAnswers[k + 1] : undefined
            let isCorrect = false
            if (userAnswer) {
              userAnswer = userAnswer.replace(regexpSym, '')
              let regAnswers = []
              angular.forEach(v, (i, j) => {
                regAnswers.push(angular.lowercase(xxtea.decryptFromBase64(i, appConstants.tea)).replace(regexpSym, ''))
              })
              isCorrect =  regAnswers.includes(angular.lowercase(userAnswer))
              if (isCorrect) {
                $scope.correctAnswers.push(k)
              }
            }
            $scope.validation[k + 1] = isCorrect
            $scope.postValidation[k] = isCorrect
            $scope.postAnswers[k] =  userAnswer || ''
          })

          let isCorrect = $scope.correctAnswers.length === $scope.answers.length
          if (isCorrect) {
            popupService.show('Very good!', 'happy-smile', true)
            $scope.correct = true
            $scope.question.answered = true
            $scope.overallCorrectAnswers++
          } else {
            if ($scope.attempt === 1) {
              popupService.show('Some of your answers are incorrect. These have been left in place.', 'semi-smile', false)
              $scope.tryAgain = true
            } else if ($scope.attempt === 2) {
              popupService.show('Some of your answers are incorrect. These have been left in place.<br>Final chance!', 'semi-smile', false)
              $scope.tryAgain = true
            } else if ($scope.attempt === 3) {
              let htmlFeedback = correctAnswers.map((v, k) => { return (k + 1) + '. ' + xxtea.decryptFromBase64(v[0], appConstants.tea) }).join('<br/>')
              popupService.show('Sorry, your answer is not correct. The correct answers were: <br/>' + htmlFeedback, 'sad-smile', false)
              $scope.correct = true
              $scope.finalAttempt = true
              $scope.question.answered = true
            }

          }
          $scope.saveAnswer($scope.question.question_id, $scope.postAnswers, $scope.postValidation)
        }
      }
    }

    /**
         * Pushes the attempt to array (Saves answer)
         * @param {int} questionId Question ID to push the attempt
         * @param {array<object>} answers The user answers array
         * @param {boolean} correct If the answer/answers is correct
         * @param {int} attemptNumber The user attempt number
         * @returns {void}
         */
    $scope.saveAnswer = (questionId, answers, correct) => {
      const attempt = {
        'question': questionId,
        'answer': answers,
        'correct': correct
      }
      $scope.allAnswers.push(attempt)
    }

    /**
         * Sends an POST request to score API and finishes the exercise
         * @returns {void}
         */
    $scope.finishExercise = () => {
      const allAnswersCount = $scope.exercise.interaction.correct.answers.length
      const score = Math.round((($scope.correctAnswers.length / allAnswersCount) * 100))
      const data = {
        'score': score,
        'answers': $scope.allAnswers,
        'timeAttempted': $scope.time_attempted,
        'timeFinished': $scope.time_finished,
        'exerciseId': $scope.exercise.exercise_id,
        'playlistId': angular.isDefined($scope.exercise.playlist.playlist_id) ? $scope.exercise.playlist.playlist_id : $localStorage.playlist_id,
        'context': $localStorage.context_name,
        'contextId': $localStorage.context_id
      }

      $http.post(appConstants.api + '/exercise/score/store', data, { headers: { 'Authorization': $scope.token } }).then(() => {
      }, $scope.errorCallback)

      let nextExercise = undefined
      if ($localStorage.playlist) {
        if ($localStorage.currentActiveExercise !== $localStorage.exerciseCount) {
          nextExercise = $localStorage.playlist[$localStorage.currentActiveExercise].exercise_id
        }
      }
      exerciseService.setVal({
        'score': score,
        'totalQuestions': allAnswersCount,
        'correctAnswer': $scope.correctAnswers.length,
        'nextExercise': nextExercise
      })
      if ($scope.loc !== 'practice') {
        $state.go('result') // Go to result page
      } else {
        $element.replaceWith($compile('<result></result>')($scope))
      }
    }

    /**
         * Retry the exercise
         * @returns {void}
         */
    $scope.try = function () {
      popupService.hide()
      $scope.validation = []
      $scope.tryAgain = false
    }

    /**
         * Checks current action that have to be displayed
         * @param {string} type text or function
         * @returns {object} string or function
         */
    $scope.currentAction = (type) => {
      if (!$scope.correct && !$scope.tryAgain) {
        return type === 'text' ? 'Check' : $scope.check()
      } else if ($scope.attempt > 0 && $scope.tryAgain) {
        return type === 'text' ? 'Try again' : $scope.try()
      } else if ($scope.correct || $scope.attempt === 3) {
        return type === 'text' ? 'Next' : $scope.nextStep()
      }
    }

    // Get exercise data and setup
    $scope.$watch('[$ctrl.questionsId]', (n, o) => {
      $scope.loc = $location.absUrl().replace('/' + $scope.location, '').split('/').slice(-1).pop()
      if ($stateParams.id) {
        $http.get(appConstants.api + '/exercise/' + $scope.$ctrl.questionsId, { headers: { 'Authorization': $scope.token } })
          .then((response) => {
            setupExercise(response.data)
            $localStorage.$reset()
            $localStorage.exercises = []
            $localStorage.questionCount = $scope.exercise.interaction.questions.length
            $localStorage.questions = $scope.exercise.interaction.questions
            $localStorage.currentActiveExercise = 1
            $localStorage.currentActiveQuestion = $scope.step + 1

            // Assign new data
            $localStorage.playlist = [response.data]
            $localStorage.exerciseCount = $localStorage.playlist.length
            $localStorage.currentExercise = response.data.exercise_id
            $localStorage.currentActiveExercise = 1
            $localStorage.questionLink = { type: 'cloze', id: $stateParams.id }
            if ($scope.loc !== 'practice') {
              $timeout(() => { $state.go('exercise') })
            }
          }, errorCallback)
      } else {
        setupExercise($localStorage.playlist.find((item) => item.exercise_id === $localStorage.currentExercise))
      }
    })
  }

}
export default clozeComponent
