import quizTemplate from './quiz.template.html'

/* @ngInject */
let quizComponent = {
  template: quizTemplate,
  bindings: {
    questionsId: '=',
    sid: '='
  },
  controller: ($rootScope, $document, $scope, $compile, $element, $http, $location, $state, $stateParams, $timeout, exerciseService, $log, popupService, $cookies, appConstants, $window, $localStorage, xxtea) => {

    $scope.token = 'Bearer ' + $cookies.get('API_TOKEN')
    /**
         * Error callback for HTTP response
         * @param {object} response The HTTP response object
         * @returns {void}
         */
    function errorCallback(response) {
      if (response.status === 401) {
        $log.error({ status: 'false', data: 'Bad token.' })
      } else if (response.status === 404) {
        $log.error({ status: 'false', data: 'Bad request.' })
      } else if (response.status === 405) {
        $log.error({ status: 'false', data: 'Method not allowed.' })
      } else {
        $log.error({ status: 'false', data: response.statusText })
      }
    };

    function getFeedback(feedback, correct) {
      if (angular.isDefined(feedback)) {
        return feedback
      }
      else {
        return correct ? 'Very good!' : 'Sorry, your answer is not correct.'
      }
    }

    $scope.toggleLayer = (element) => {
      angular.element($document[0].querySelector('#' + element)).toggleClass('hidden')
    }

    $scope.toggleaudio = (element) => {
      angular.element($document[0].querySelector('#' + element)).toggleClass('hidden')
    }
    $window.toggleLayer = $scope.toggleLayer
    $window.toggleaudio = $scope.toggleaudio
    $scope.step = 0
    $scope.current = {
      selectedIndex: undefined,
      selectedIndexes: [],
      isCorrect: undefined
    }
    $scope.attempt = 0
    $scope.correct = false
    $scope.queans = {}
    $scope.tryAgain = false
    $scope.finalAttempt = false
    $scope.correctAnswerCount = 0
    $scope.score = 0
    $scope.time_finished = ''
    $scope.time_attempted = Math.round(new Date().getTime() / 1000)
    $scope.currentActiveQuestion = $scope.step + 1
    $localStorage.currentActiveQuestion = $scope.step + 1


    $scope.$watch('step', (n, o) => {
      popupService.hide()
      $scope.current.selectedIndex = undefined
      $scope.current.selectedIndexes = []
      $scope.current.isCorrect = undefined
      $scope.correct = false
      $scope.attempt = 0
      $scope.tryAgain = false
      $scope.finalAttempt = false
      $scope.attempts = []
      $scope.disableText = false

      refreshSelectorData();
    })

    $scope.nextStep = () => {
      if ($scope.step + 1 === $scope.exercise.interaction.questions.length) {
        $scope.time_finished = Math.round(new Date().getTime() / 1000)
        $scope.finishExercise()
      } else {
        popupService.hide()
        $scope.step++
        $scope.current.selectedIndex = undefined
        $scope.current.selectedIndexes = []
        $scope.current.isCorrect = undefined
        $scope.correct = false
        $scope.attempt = 0
        $scope.tryAgain = false
        $scope.finalAttempt = false
        $scope.attempts = []
        $scope.disableText = false
        $scope.current.selectedAnswer = ''
      }

      $scope.currentActiveQuestion = $scope.step + 1
      $localStorage.currentActiveQuestion = $scope.step + 1
    }


    // Check Answer.
    $scope.check = (question) => {
      // Setup variables
      const regexpSym = /([.'"`.,|()!?:;-])/g
      let feedback = undefined
      let answerId = undefined
      let userAnswerSa = angular.lowercase($scope.current.selectedAnswer) || ''
      let userAnswerMc = $scope.exercise.interaction.questions[$scope.step].answers[$scope.current.selectedIndex] || ''
      let userAnswerMs = $scope.exercise.interaction.questions[$scope.step].answers[$scope.current.selectedIndexes[0]] || ''
      $scope.postUserAnswerMs = ['']
      angular.forEach($scope.current.selectedIndexes, (v, k) => {
        $scope.postUserAnswerMs[k] = $scope.exercise.interaction.questions[$scope.step].answers[v].answer_id
      })
      let cQuestion = $scope.exercise.interaction.correct.find((q) => { return q.question_id === question.question_id })
      if ($scope.exercise.interaction.questions[$scope.step].answers.length <= 3) {
        $scope.possibleAttemptsCount = $scope.exercise.interaction.questions[$scope.step].answers.length - 1
      } else {
        $scope.possibleAttemptsCount = 3
      }
      // Validation
      if (cQuestion) {
        $scope.attempt++
        if (question.question_type === 'sa') {
          if (question.answered === true) {
            popupService.show('You have already answered this question!', 'semi-smile', false)
          } else {
            let foundedQuestion = question.answers.find((q) => { return q.answer_text === userAnswerSa ? userAnswerSa : '' })
            if (foundedQuestion) { feedback = foundedQuestion.feedback; answerId = foundedQuestion.answer_id }
            if (userAnswerSa.feedback !== '') {
              feedback = xxtea.decryptFromBase64(userAnswerSa.feedback, appConstants.tea)
            }
            userAnswerSa = userAnswerSa.replace(regexpSym, '')
            let regAnswers = []
            angular.forEach(cQuestion.answers, (v, k) => {
              regAnswers.push(angular.lowercase(xxtea.decryptFromBase64(v, appConstants.tea)).replace(regexpSym, ''))
            })
            let isCorrect = regAnswers.includes(userAnswerSa)
            $scope.disableText = true
            $scope.saveAnswer(question.question_id, userAnswerSa, [isCorrect])
            if (isCorrect) {
              question.answered = true
              popupService.show(getFeedback(feedback, true), 'happy-smile', true)
              $scope.correct = true
              $scope.correctAnswerCount++
            } else {
              let correctAnswers = question.answers.filter((q) => cQuestion.answers.includes(xxtea.encryptToBase64(q.answer_text, appConstants.tea))).map((q) => q.answer_text)
              $scope.correctAnswers = correctAnswers;
              $scope.tryAgain = $scope.attempt < 3
              $scope.correct = $scope.attempt > 2
              $scope.finalAttempt = $scope.attempt > 2

              if ($scope.attempt === 1) {
                popupService.show(getFeedback(feedback, false), 'semi-smile', false)
              } else if ($scope.attempt === 2) {
                popupService.show(getFeedback(feedback, false) + '<br/>Final chance!', 'semi-smile', false)
              } else if ($scope.attempt === 3) {
                question.answered = true
                popupService.show(getFeedback(feedback, false) + '<br/>The correct answer is:<br/>' + correctAnswers.join(',<br>'), 'sad-smile', false)
              }
            }
          }
        } else if (question.question_type === 'mc') {
          $scope.postAnswers = []
          $scope.postAnswers[0] =  userAnswerMc.answer_id || ''
          if (question.answered === true) {
            popupService.show('You have already answered this question!', 'semi-smile', false)
          } else {
            let foundedQuestion = question.answers.find((q) => { return q.answer_text === userAnswerMc ? userAnswerMc.answer_text : '' })
            if (foundedQuestion) { feedback = foundedQuestion.feedback; answerId = foundedQuestion.answer_id }
            if (userAnswerMc.feedback !== '') {
              feedback = xxtea.decryptFromBase64(userAnswerMc.feedback, appConstants.tea)
            }
            let isCorrect = cQuestion.answers.includes(xxtea.encryptToBase64(userAnswerMc ? userAnswerMc.answer_id : '', appConstants.tea))
            $scope.current.isCorrect = isCorrect ? 'yes' : 'no'
            $scope.disableText = true
            if (isCorrect) {
              question.answered = true
              popupService.show(getFeedback(feedback, true), 'happy-smile', true)
              $scope.correct = true
              $scope.correctAnswerCount++
            } else {
              $scope.tryAgain = $scope.attempt < $scope.possibleAttemptsCount
              $scope.correct = $scope.attempt > $scope.possibleAttemptsCount - 1
              $scope.finalAttempt = $scope.attempt > $scope.possibleAttemptsCount - 1

              $scope.correctAnswers = question.answers.filter((q) => cQuestion.answers.includes(xxtea.encryptToBase64(q.answer_id, appConstants.tea))).map((q) => q.answer_text);
              if ($scope.attempt === 1) {
                popupService.show(getFeedback(feedback, false), 'semi-smile', false)
              } else if ($scope.attempt === $scope.possibleAttemptsCount - 1) {
                popupService.show(getFeedback(feedback, false) + '<br/>Final chance!', 'semi-smile', false)
              } else {
                question.answered = true
                let correctAnswers = question.answers.filter((q) => cQuestion.answers.includes(xxtea.encryptToBase64(q.answer_id, appConstants.tea))).map((q) => q.answer_text)
                popupService.show(getFeedback(feedback, false) + '<br/>The correct answer is:<br/>' + correctAnswers.join(',<br>'), 'sad-smile', false)
              }
            }
            $scope.saveAnswer(question.question_id, [userAnswerMc !== '' ? userAnswerMc.answer_id : ''], [isCorrect])
          }
        } else {
          if (question.answered === true) {
            popupService.show('You have already answered this question!', 'semi-smile', true)
          } else {
            let foundedQuestion = question.answers.find((q) => { return q.answer_id === userAnswerMs ? userAnswerMs.answer_id : '' })
            if (foundedQuestion) { feedback = foundedQuestion.feedback; answerId = foundedQuestion.answer_id }
            if (userAnswerMs.feedback !== '') {
              feedback = xxtea.decryptFromBase64(userAnswerMs.feedback, appConstants.tea)
            }
            let userCorrectAnswers = $scope.current.selectedIndexes.filter((v, k) => {
              let answer = $scope.exercise.interaction.questions[$scope.step].answers[v]
              let encrypted = xxtea.encryptToBase64(answer.answer_id, appConstants.tea)
              return cQuestion.answers.includes(encrypted)
            })
            let isCorrect = userCorrectAnswers.length === cQuestion.answers.length && $scope.current.selectedIndexes.length === cQuestion.answers.length
            $scope.postCorrectAnswers = [false]
            angular.forEach($scope.current.selectedIndexes, (v, k) => {
              let answer = $scope.exercise.interaction.questions[$scope.step].answers[v]
              let encrypted = xxtea.encryptToBase64(answer.answer_id, appConstants.tea)
              $scope.postCorrectAnswers[k] = cQuestion.answers.includes(encrypted)
            })

            $scope.current.isCorrect = isCorrect ? 'yes' : 'no'
            $scope.disableText = true
            if (isCorrect) {
              question.answered = true
              popupService.show(getFeedback(feedback, true), 'happy-smile', true)
              $scope.correct = true
              $scope.correctAnswerCount++
            } else {
              let firstNotCorrectAnswer = $scope.exercise.interaction.questions[$scope.step].answers.find((v, k) => {
                let encrypted = xxtea.encryptToBase64(v.answer_id, appConstants.tea)
                return !cQuestion.answers.includes(encrypted)
              })
              if (firstNotCorrectAnswer) { feedback = xxtea.decryptFromBase64(firstNotCorrectAnswer.feedback, appConstants.tea); answerId = firstNotCorrectAnswer.answer_id }

              $scope.tryAgain = $scope.attempt < 3
              $scope.correct = $scope.attempt > 2
              $scope.finalAttempt = $scope.attempt > 2
              $scope.correctAnswers = question.answers.filter((q) => cQuestion.answers.includes(xxtea.encryptToBase64(q.answer_id, appConstants.tea))).map((q) => q.answer_text);
              if ($scope.attempt === 1) {
                popupService.show(getFeedback(feedback, false), 'semi-smile', false)
              } else if ($scope.attempt === 2) {
                popupService.show(getFeedback(feedback, false), 'semi-smile', false)
              } else {
                question.answered = true
                let correctAnswers = question.answers.filter((q) => cQuestion.answers.includes(xxtea.encryptToBase64(q.answer_id, appConstants.tea))).map((q) => q.answer_text)
                popupService.show(getFeedback(feedback, false) + '<br/>The correct answer is:<br/>' + correctAnswers.join(',<br>'), 'sad-smile', false)
              }
            }
            $scope.saveAnswer(question.question_id, $scope.postUserAnswerMs, $scope.postCorrectAnswers)
          }
        }
      }
    }

    $scope.answers = []
    $scope.attempts = []

    $scope.saveAnswer = (questionId, answerIds, correct) => {

      if (angular.isArray(answerIds)) {
        var answerIds = answerIds.map(x => parseInt(x, 10))
      }

      const attempt = {
        'answer': answerIds,
        'correct': correct,
      }

      if ($scope.answers.length > 0) {
        let matchQuestionId = ''
        angular.forEach($scope.answers, obj => {
          if (obj.question === questionId) {
            matchQuestionId = obj
          }
        })
        if (matchQuestionId) {
          matchQuestionId.attempts.push(attempt)
        } else {
          $scope.attempts.push(attempt)

          const dataObj = {
            'question': questionId,
            'attempts': $scope.attempts
          }

          $scope.answers.push(dataObj)
        }
      } else {
        $scope.attempts.push(attempt)

        const data = {
          'question': questionId,
          'attempts': $scope.attempts
        }

        $scope.answers.push(data)
      }
    }

    // reset view while Try Again.
    $scope.try = () => {
      popupService.hide()
      $scope.disableText = false
      $scope.current.selectedIndex = undefined
      $scope.current.selectedIndexes = []
      $scope.current.isCorrect = undefined
      $scope.tryAgain = false
      $scope.current.selectedAnswer = ''
    }


    /******************************* NEW ***************************************************************************************************** */
    /**
         * Sends an POST request to score API and finishes the exercise
         * @returns {void}
         */
    $scope.finishExercise = () => {
      let allAnswersCount = 0
      let correctAnswersCount = 0 // Correct answers counter
      // Get correct answers count
      angular.forEach($scope.exercise.interaction.correct, (correct) => { allAnswersCount += correct.answers.length })
      angular.forEach($scope.answers, (answer) => { correctAnswersCount += answer.attempts[answer.attempts.length - 1].correct[0].length >= 0 ?
        answer.attempts[answer.attempts.length - 1].correct.length : answer.attempts[answer.attempts.length - 1].correct[0] ? 1 : 0 })
      // or $scope.exercise.interaction.questions.map((q) => { return q.answers.length; }).reduce((a, b) => { return a + b; });
      const score = Math.round(((correctAnswersCount / allAnswersCount) * 100))
      const data = {
        'score': score,
        'answers': $scope.answers,
        'timeAttempted': $scope.time_attempted,
        'timeFinished': $scope.time_finished,
        'exerciseId': $scope.exercise.exercise_id,
        'playlistId': angular.isDefined($scope.exercise.playlist.playlist_id) ? $scope.exercise.playlist.playlist_id : $localStorage.playlist_id,
        'context': $localStorage.context_name,
        'contextId': $localStorage.context_id
      }

      $http.post(appConstants.api + '/exercise/score/store', data, { headers: { 'Authorization': $scope.token } }).then(() => {
      }, $scope.errorCallback)
      let nextExercise = undefined
      if ($localStorage.playlist) {
        if ($localStorage.currentActiveExercise !== $localStorage.exerciseCount) {
          nextExercise = $localStorage.playlist[$localStorage.currentActiveExercise].exercise_id
        }
      }

      exerciseService.setVal({
        'score': score,
        'totalQuestions': allAnswersCount,
        'correctAnswer': correctAnswersCount,
        'nextExercise': nextExercise
      })

      if ($scope.loc !== 'practice') {
        $state.go('result') // Go to result page
      } else {
        $element.replaceWith($compile('<result></result>')($scope))
      }

    }

    /**
         * Checks current action that have to be displayed
         * @param {string} type text or function
         * @returns {object} string or function
         */
    $scope.currentAction = (type) => {
      if (!$scope.correct && !$scope.tryAgain) {
        return type === 'text' ? 'Check' : $scope.check($scope.exercise.interaction.questions[$scope.step])
      } else if ($scope.attempt > 0 && $scope.tryAgain) {
        return type === 'text' ? 'Try again' : $scope.try()
      } else if ($scope.correct || $scope.attempt === $scope.possibleAttemptsCount) {
        return type === 'text' ? 'Next' : $scope.nextStep()
      }
    }

    /**
         * Setting up the exercise
         * @param {object} exerciseData exercise object
         * @returns {void}
         */
    function setupExercise(exerciseData) {
      $scope.exercise = exerciseData
      $scope.question = $scope.exercise.interaction.questions[$scope.step]
      $localStorage.questionCount = $scope.exercise.interaction.questions.length // Get questions count
      refreshSelectorData();
    }

    function refreshSelectorData() {
      if (angular.isDefined($scope.exercise)) {
        $scope.correctAnswersCount = $scope.exercise.interaction.correct.find((q) => { return q.question_id === $scope.exercise.interaction.questions[$scope.step].question_id }).answers.length;
        if ($scope.exercise.interaction.questions[$scope.step].answers.length <= 3) {
          $scope.possibleAttemptsCount = $scope.exercise.interaction.questions[$scope.step].answers.length - 1
        } else {
          $scope.possibleAttemptsCount = 3
        }
      }
    }

    // Get exercise data and setup
    $scope.$watch('[$ctrl.questionsId]', (n, o) => {
      $scope.loc = $location.absUrl().replace('/' + $scope.location, '').split('/').slice(-1).pop()
      if ($stateParams.id) {
        $http.get(appConstants.api + '/exercise/' + $scope.$ctrl.questionsId, { headers: { 'Authorization': $scope.token } })
          .then((response) => {
            setupExercise(response.data)
            $localStorage.$reset()
            $localStorage.exercises = []
            $localStorage.questionCount = $scope.exercise.interaction.questions.length
            $localStorage.questions = $scope.exercise.interaction.questions
            $localStorage.currentActiveExercise = 1
            $localStorage.currentActiveQuestion = $scope.step + 1

            // Assign new data
            $localStorage.playlist = [response.data]
            $localStorage.exerciseCount = $localStorage.playlist.length
            $localStorage.currentExercise = response.data.exercise_id
            $localStorage.currentActiveExercise = 1
            $localStorage.questionLink = { type: 'quiz', id: $scope.$ctrl.questionsId }
            if ($scope.loc !== 'practice') {
              $timeout(() => { $state.go('exercise') })
            }
          }, errorCallback)
      } else {
        setupExercise($localStorage.playlist.find((item) => item.exercise_id === $localStorage.currentExercise))
      }
    })
  }

}
export default quizComponent
