import webpack from 'webpack';
import WebpackConfig from 'webpack-config';
import path from 'path';
import CleanWebpackPlugin from 'clean-webpack-plugin';
const ExtractTextPlugin = require('extract-text-webpack-plugin')

const plugins = [
  new ExtractTextPlugin({
    filename: 'theme.css'
  }),
  new CleanWebpackPlugin(['dist'], {
    root: __dirname,
    verbose: true,
    dry: false
  })
]

const config = {
  output: {
    path: path.join(__dirname, '/dist'),
    filename: 'snippet.js'
  },
  context: path.join(__dirname, '/app'),
  module: {
    loaders: [
      {
        test: /\.scss$/,
        use: ExtractTextPlugin.extract({
          fallback: 'style-loader',
          use: ['css-loader', 'sass-loader']
        })
      },
      {
        test: /\.(eot|woff|woff2|ttf|png|svg|jpg)$/,
        loader: 'url-loader?limit=100000&name=[hash].[ext]',
      },
      {
        test: /\.json$/,
        loader: 'json-loader'
      },
      {
        test: /\.html$/,
        loader: 'ng-cache-loader?prefix=[dir]/[dir]'
      },
      {
        test: /\.js$/,
        exclude: [/node_modules/, /assets/],
        loaders: ['babel-loader', 'eslint-loader']
      }
    ]
  },
  plugins: plugins
}

module.exports = config;
